;; -*- mode: scheme; coding: utf-8 -*-

;;;; Copyright (C) 2011, 2012, 2013, 2014
;;;; Free Software Foundation, Inc.
;;;;
;;;; This library is free software; you can redistribute it and/or
;;;; modify it under the terms of the GNU General Public License as
;;;; published by the Free Software Foundation; either version 3 of
;;;; the License, or (at your option) any later version.
;;;; 
;;;; This library is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;;; General Public License for more details.
;;;; 
;;;; You should have received a copy of the GNU General Public License
;;;; along with this library; if not, write to the Free Software
;;;; Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
;;;; 02110-1301 USA
;;;;

;;; Commentary:

;;; Code:

(define-module (fs fs-ops)
  :use-module (ice-9 popen)
  :use-module (ice-9 rdelim)
  :use-module (ice-9 receive)
  :use-module (ice-9 ftw)
  :use-module (srfi srfi-1)
  ;; :use-module (macros reexport)
  :use-module (macros push)
  :use-module (macros do)
  :use-module (lists lists)
  :use-module (strings strings)
  :use-module (support slib)
  :export (get-filenames
	   split-filename
	   pair-files
	   mk-dir
	   cp-files
	   rm-files
	   cat-file
	   is-more-recent?))


#;(eval-when (compile load eval)
  (re-export-public-interface (ice-9 popen)
			      (ice-9 rdelim)
			      (ice-9 receive)
			      (ice-9 ftw)
			      (srfi srfi-1)
			      (macros push)
			      (macros do)
			      (lists lists)
			      (strings strings)
			      (support slib)))

(define* (split-filename filename #:key (dot-lindex #f))
  (let* ((d-name (dirname filename))
	 (b-name (basename filename))
	 (dot (if dot-lindex
		  (string-index b-name #\.)
		  (string-rindex b-name #\.)))
	 (f-name (if dot (substring b-name 0 dot) b-name))
	 (f-ext (if dot (substring b-name (1+ dot)) #f)))
    (values d-name b-name f-name f-ext)))

(define* (get-filenames in #:key (like #f) (ci #f) (recursively #f) (dir-assoc #f))
  (let ((d-assoc (list))
	(files (list))
	(dir-nb 0)
	(file-nb 0)
	(matcher (and like
		      (if ci
			  (glob:make-matcher like char-ci=? char-ci<=?)
			  (glob:make-matcher like char=? char<=? )))))
    (nftw in
	  (lambda (filename statinfo flag base level)
	    (case flag
	      (#;(regular symlink)
	       (regular)
	       (if (or recursively (= level 1))
		   (receive (d-name b-name f-name f-ext)
		       (split-filename filename)
		     (when (or (not matcher)
			       (matcher b-name))
		       (unless dir-assoc (push! filename files))
		       (set! file-nb (1+ file-nb))
		       (let ((values (assoc-ref d-assoc d-name)))
			 (if values
			     (set! d-assoc (assoc-set! d-assoc d-name (cons b-name values)))
			     (begin
			       (set! dir-nb (1+ dir-nb))
			       (set! d-assoc (assoc-set! d-assoc d-name (list b-name)))))))))))
	    #t) ;; do not stop
	  'physical)
    (if dir-assoc
	(values d-assoc dir-nb file-nb)
	(values (reverse! files) dir-nb file-nb))))

(define (pair-files files a b)
  (if (< (length files) 2)
      files
      (let* ((a-files (sort (filter-map (lambda (file) (and (string-contains file a) file)) files) string<?))
	     (no-a-files (filter-map (lambda (file) (str/delete file a)) a-files))
	     (b-files (sort (filter-map (lambda (file) (and (string-contains file b) file)) files) string<?))
	     (no-b-files (filter-map (lambda (file) (str/delete file b)) b-files))
	     (paired-files (list)))
	(dotimes (i (length no-a-files) paired-files)
	  (let ((pos (list-pos (list-ref no-a-files i) no-b-files string=?)))
	    (when pos (push! (cons (list-ref a-files i) (list-ref b-files pos)) paired-files)))))))

(define (mk-dir path)
  ;; the builtin mkdir guile core function calls the operating system
  ;; one, not the GNU Linux shell script, but it does not implement
  ;; the '-p' option, which is essential for us.  here is a first, quick
  ;; and dirty [it always apply '-p'] implementation, we'll review it
  ;; later.
  (let* ((cmd (format #f "mkdir -p ~A" path))
	 (s (open-input-pipe cmd))
	 (results (read-line s)))
    (if (zero? (status:exit-val (close-pipe s)))
	#t
	(error "subprocess returned non-zero result code" cmd))))

(define (cp-files files from to)
  (for-each (lambda (f-bname)
	      (copy-file (string-append from "/" f-bname) (string-append to "/" f-bname)))
      files))

(define* (rm-files in #:key (like #f) (ci #f) (recursively #f))
  (receive (filenames d-nb f-nb)
      (get-filenames in #:like like #:ci ci #:recursively recursively)
    (for-each (lambda (filename) (delete-file filename)) filenames)))

(define* (cat-file a b #:key (mode ">>"))
  (let* ((cmd (format #f "cat ~A ~A ~A" a mode b))
	 (s (open-input-pipe cmd))
	 (results (read-line s)))
    (if (zero? (status:exit-val (close-pipe s)))
	#t
	(error "subprocess returned non-zero result code" cmd))))

(define (is-more-recent? filename1 filename2)
  (let ((stat1 (stat filename1))
	(stat2 (stat filename2)))
    (or (> (stat:mtime stat1) (stat:mtime stat2))
        (and (= (stat:mtime stat1) (stat:mtime stat2))
             (>= (stat:mtimensec stat1)
                 (stat:mtimensec stat2))))))



#!

(use-modules (fs fs-ops))
(reload-module (resolve-module '(fs fs-ops)))

!#
